<?php
namespace Controllers; 

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification; 

class Ger80200ApuracaoDespesasController extends \Teknisa\Libs\Controller\CRUD{
    
    protected $dataSourceNameTipoopeacRop   =  "tipooperacrop";
    protected $dataSourceNameCtPagar        =  "tipoctpagrop";
    protected $dataSourceManager;
    protected $connection;
    protected $environment;
    
    public function __construct($crudService, $environment, $connection) {
        $this->connection = $connection;
        $this->environment = $environment;
        parent::__construct($crudService, $environment);
    }
    
    /**
    * salvaDespesasDMI
    *
    * Salva o tipo de conta a pagar em um tipo de despesa, e
    * antes confere se essa já foi cadastrada como outro tipo
    * de despesas. Sendo o caso, ele mostra uma mensagem informando
    * onde a conta já foi cadastrada.
    * 
    * 
  
    * @param string
    * @param integer CDFILIAL
    * @param integer CDTPCTPAGAR
    * 
    * @return mensagem (ou de salvo, ou de onde a conta já está salva)
    */
    
    public function saveTipooper (DTO\Request $request, DTO\Response $response){

        $dados         = $request->getParameter("DataSet");
        
        $tipooper      = $dados[0]['CDTIPOOPER'];
        $cdfilial      = $dados[0]['CDFILIAL'];
        $idtpoperacrop = $dados[0]['IDTPOPERACROP'];
        $identrsaidop  = $dados[0]['IDENTRSAIDOP'];
        $nrorg         = $dados[0]['NRORG'];
        
        $messages      = [];
        $rowToSave     = [];
        
        if($dados != null){
            
            foreach($dados[0]['CDTIPOOPER'] as $itemTipooper){
                
                /*Pego a posição do tipo de operação no array e guardo na variavel key*/
                $key = array_search($itemTipooper,$dados[0]['CDTIPOOPER']);
                
                /*Sabendo a posição do tipo de operação(key) no array de nomes, pego o nome do item na posição correspondente*/
                $nmtipooper = $dados[0]['NMTIPOOPER'][$key];
                
                $filtro = array(
                    array(
                        "columnName" => "CDFILIAL",
                        "operator"   => "=",
                        "value"      => $cdfilial
                    ),
                    array(
                        "columnName" => "CDTIPOOPER",
                        "operator"   => "=",
                        "value"      => $itemTipooper
                    ),
                    array(
                        "columnName" => "IDTPOPERACROP",
                        "operator"   => "=",
                        "value"      => $idtpoperacrop
                    ),
                    array(
                        "columnName" => "IDENTRSAIDOP",
                        "operator"   => "=",
                        "value"      => $identrsaidop
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator"   => "=",
                        "value"      => $nrorg
                    )
                );
                
                $dataSet = $this   ->crudService->getFilteredDataSet($this->dataSourceNameTipoopeacRop, $filtro);
                $array   = $dataSet->getRows();
                
                if(!empty($array)){
                
                    array_push($messages,"Tipo de Operação já cadastrado: ".$array[0]['CDTIPOOPER']." - ".$nmtipooper);
                    
                }else{
                    $row["__is_new"]        = true;
                    $row['CDFILIAL']        = $cdfilial;
                    $row['CDTIPOOPER']      = $itemTipooper;
                    $row['IDENTRSAIDOP']    = $identrsaidop;
                    $row['IDTPOPERACROP']   = $idtpoperacrop;
                    $row['NRORG']           = $nrorg;
                    $row['NRORGULTATU']     = $nrorg;
                    $row['IDCOMPFATBRUTO']  = 'N';
                    $row['DTULTATU']        = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
                    $row['DTULTATUTPOR']    = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
	                $row['DTINCLUSAO']      = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
	                $row['CDOPERINCLUSAO']  = $this->environment->getCdOperador();
	                $row['CDOPERULTATU']    = $this->environment->getCdOperador();;
	                $row['NRORGINCLUSAO']   = $this->environment->getNrorg();
                    $rowToSave[]            = $row;
                }
            }

            if($rowToSave != null){
                
                try{
                    $this->crudService->save($this->dataSourceNameTipoopeacRop, $rowToSave);
                    $saved = true;
                }catch(\Exception $ex){
                    throw new \Exception("Erro ao tentar inserir na base de dados, verifique os parametros e tente novamente!" . $ex->getMessage());
                }
            }
            
            $response->addDataSet(new DataSet("saved", [$saved]));
            
            if(!empty($messages)){
                $response->addDataSet(new DataSet("errorList", $messages));
                throw new \Exception ("Ocorreram erros durante a operação. Verifique o log");
            }
        }
    }
    
    public function saveDespesaProduto (DTO\Request $request, DTO\Response $response ){
        $rows           = $request->getDataSet()->getRows();
        $erroList       = array();
        $saved          = false;
        
        if($rows != null){
            $cdfilial       = $rows[0]['CDFILIAL'];
            $idtpprodutorop = $rows[0]['IDTPPRODUTOROP'];
            $nrorg          = $rows[0]['NRORG'];
            $operador       = $rows[0]['CDOPERADOR'];
            
            switch ($idtpprodutorop) {
                case 'DE':
                    $tipo ="Despesas Despesas Diversas";
                    break;
                case 'EX':
                    $tipo ="Exceção em Despesas Diversas";
                    break;
                case 'MO':
                    $tipo ="Mão de Obra";
                    break;
                case 'MX':
                    $tipo ="Exceção em Mão de Obra";
                    break;
                case 'CO':
                    $tipo ="Entrada por Compras";
                case 'OX':
                    $tipo ="Exceçao em Entrada por Compras";
            }
                
            foreach($rows[0]["CDPRODUTO"] as $itemProd) {
                /*Pego a posição do produto e guardo na variavel key*/
                $key = array_search($itemProd,$rows[0]['CDPRODUTO']);
                
                /*Sabendo a posição do produto (key) no array de nomes, pego o nome do item na posição correspondente*/
                $nmproduto = $rows[0]['NMPRODUTO'][$key];
                $cdarvprod = $rows[0]['CDARVPRODD'][$key];

                try{
                    $validProd = $this->validaProduto($cdfilial, $itemProd, $cdarvprod, $nmproduto ,$tipo, $idtpprodutorop);
                    
                    if(empty($validProd)){
                        $row['__is_new']       = true;
                        $row['CDFILIAL']       = $cdfilial;
                        $row['CDPRODUTO']      = $itemProd;
                        $row['IDTPPRODUTOROP'] = $idtpprodutorop;
                        $row['CDPRODUTO']      = $itemProd;
                        $row['IDTPPRODUTOROP'] = $idtpprodutorop;
                        $row['DTULTATUPRRP']   = date('d/m/Y');
                        $row['DTINCLUSAO']     = date('d/m/Y');
                        $row['DTULTATU']       = date('d/m/Y');
                        $row['NRORG']          = $nrorg;
                        $row['NRORGINCLUSAO']  = $nrorg;
                        $row['NRORGULTATU']    = $nrorg;
                        $row['CDOPERINCLUSAO'] = $operador;
                        $row['CDOPERULTATU']   = $operador;
                        $row['IDATIVO']        = 'S';
        
                        $this->crudService->save('produtorop', [$row]);
                        $saved = true;
                    } else{
                        foreach($validProd as $prod){
                            array_push($erroList, $prod);
                        }
                    }
                } catch(\Exception $ex){
                    if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
                        array_push($erroList, "Nao foi possivel processar os dados enviados   --- " . $ex->getMessage());
                    } else{
                        array_push($erroList, "Produto " .$nmproduto. " já está cadastrado!");
                    }
                }
            }
            
            $response->addDataSet(new DataSet("saved", [$saved]));
            
            if(!empty($erroList)){
                $response->addDataSet(new DataSet("errorList", $erroList));
                throw new \Exception ("Ocorreram erros durante a operação. Verifique o log");
            }
        }
    }
    
    public function salvaDespesasDMI (DTO\Request $request, DTO\Response $response){
       $dados = $request->getParameter("DataSet"); 
       
       try{
            if($dados != null){
                
                $despesas = $dados[0];
                $despesas['IDATIVO']        = 'S';
                $despesas['DTULTATU']       = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
                $despesas['DTINCLUSAO']     = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
                $despesas['CDOPERINCLUSAO'] = $this->environment->getCdOperador();
                $despesas['CDOPERULTATU']   = $this->environment->getCdOperador();;
                $despesas['NRORGINCLUSAO']  = $this->environment->getNrorg();
                $despesas['NRORGULTATU']    = $this->environment->getNrorg();
                
                $filtro = array(
                    array(
                        "columnName" => "CDFILIAL",
                        "operator" => "=",
                        "value" => $despesas["CDFILIAL"]
                    ),
                    array(
                        
                        "columnName" => "CDTPCTPAGAR",
                        "operator" => "=",
                        "value" => $despesas["CDTPCTPAGAR"]
                    ),
                    array(
                        "columnName" => "NRORG",
                        "operator"   => "=",
                        "value"      => $despesas["NRORG"]
                    )
                );
            }
            
            $dataSet = $this->crudService->getFilteredDataSet($this->dataSourceNameCtPagar, $filtro);
            $array   = $dataSet->getRows();
            
            if(!empty($array)){
                
                $tipo = $array[0]["IDTPCTPAGROP"];
                $mensagem = "";
                if($tipo == "MAO"){
                    $mensagem = "Mão de Obra";
                }else if($tipo == "DSP"){
                    $mensagem = "Despesas Diversas";
                }else if($tipo == "INV"){
                    $mensagem = "Investimentos";
                }
                throw new \Exception("TIPOCONTAJACAD ".$mensagem );
            }

           $rowToSave[] = $despesas;  
           
            try{
                $this->crudService->save($this->dataSourceNameCtPagar, $rowToSave);
            }catch(\Exception $ex){
                throw new \Exception("ERROAOINSERIRDADOS " . $ex->getMessage());
            }
            
        }catch(\Exception $ex){
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function validaProduto($cdfilial, $cdProduto, $cdarvprod, $nmproduto, $tipo, $idtpprodutorop){
        
        /*$cdProduto e $nmproduto são os produtos que serao salvos. Em caso da query
        retornar resultado, serão usados para compor a mensagem de retorno*/
        
        $duplicatedProd = array();
        $parametros     = array(':P_CDFILIAL'  => $cdfilial , ':P_CDARVPROD' => $cdarvprod);
        $parametrosEx   = array(':P_CDFILIAL'  => $cdfilial , ':P_CDPRODUTO' => $cdProduto,':P_IDTPPRODUTOROP'  => $idtpprodutorop);
                      
        if ($idtpprodutorop ==='DE' || $idtpprodutorop ==='MO' || $idtpprodutorop ==='CO'){
            
             $stmtDE = $this->connection->prepare("SELECT PR.CDPRODUTO, P.NMPRODUTO, 'NIVELANT' AS TIPO
                                                     FROM PRODUTOROP PR,PRODUTO P
                                                    WHERE PR.CDPRODUTO      = P.CDPRODUTO 
                                                      AND PR.CDFILIAL       = :P_CDFILIAL
                                                      AND PR.IDTPPRODUTOROP = 'DE'
                                                      AND P.CDARVPROD LIKE :P_CDARVPROD||'%'
                                                    UNION ALL                                              
                                                   SELECT PR.CDPRODUTO, P.NMPRODUTO, 'NIVELPOS' AS TIPO
                                                     FROM PRODUTOROP PR,PRODUTO P
                                                    WHERE PR.CDPRODUTO      = P.CDPRODUTO 
                                                      AND PR.CDFILIAL       = :P_CDFILIAL
                                                      AND PR.IDTPPRODUTOROP = 'DE'
                                                      AND :P_CDARVPROD LIKE  P.CDARVPROD||'%'
                                                      AND NOT EXISTS (SELECT PPR.CDFILIAL
                                                                        FROM PRODUTOROP PPR, PRODUTO PP
                                                                       WHERE PPR.IDTPPRODUTOROP = 'EX'
                                                                         AND PPR.CDFILIAL = PR.CDFILIAL
                                                                         AND PPR.CDPRODUTO = PP.CDPRODUTO
                                                                         AND :P_CDARVPROD LIKE PP.CDARVPROD||'%')"); 
          
             $stmtMO = $this->connection->prepare("SELECT PR.CDPRODUTO, P.NMPRODUTO, 'NIVELANT' AS TIPO
                                                     FROM PRODUTOROP PR,PRODUTO P
                                                    WHERE PR.CDPRODUTO      = P.CDPRODUTO 
                                                      AND PR.CDFILIAL       = :P_CDFILIAL
                                                      AND PR.IDTPPRODUTOROP = 'MO'
                                                      AND P.CDARVPROD LIKE :P_CDARVPROD||'%'
                                                    UNION ALL                                              
                                                   SELECT PR.CDPRODUTO, P.NMPRODUTO, 'NIVELPOS' AS TIPO
                                                     FROM PRODUTOROP PR,PRODUTO P
                                                    WHERE PR.CDPRODUTO      = P.CDPRODUTO 
                                                      AND PR.CDFILIAL       = :P_CDFILIAL
                                                      AND PR.IDTPPRODUTOROP = 'MO'
                                                      AND :P_CDARVPROD LIKE  P.CDARVPROD||'%'
                                                      AND NOT EXISTS (SELECT PPR.CDFILIAL
                                                                        FROM PRODUTOROP PPR, PRODUTO PP
                                                                       WHERE PPR.IDTPPRODUTOROP = 'MX'
                                                                         AND PPR.CDFILIAL = PR.CDFILIAL
                                                                         AND PPR.CDPRODUTO = PP.CDPRODUTO
                                                                         AND :P_CDARVPROD LIKE PP.CDARVPROD||'%')");
                                                      
            $stmtCO = $this->connection->prepare("SELECT PR.CDPRODUTO, P.NMPRODUTO, 'NIVELANT' AS TIPO
                                                    FROM PRODUTOROP PR,PRODUTO P
                                                   WHERE PR.CDPRODUTO      = P.CDPRODUTO 
                                                     AND PR.CDFILIAL       = :P_CDFILIAL
                                                     AND PR.IDTPPRODUTOROP = 'CO'
                                                     AND P.CDARVPROD LIKE :P_CDARVPROD||'%'
                                                   UNION ALL                                              
                                                  SELECT PR.CDPRODUTO, P.NMPRODUTO, 'NIVELPOS' AS TIPO
                                                    FROM PRODUTOROP PR,PRODUTO P
                                                   WHERE PR.CDPRODUTO      = P.CDPRODUTO 
                                                     AND PR.CDFILIAL       = :P_CDFILIAL
                                                     AND PR.IDTPPRODUTOROP = 'CO'
                                                     AND :P_CDARVPROD LIKE  P.CDARVPROD||'%'
                                                     AND NOT EXISTS (SELECT PPR.CDFILIAL
                                                                       FROM PRODUTOROP PPR, PRODUTO PP
                                                                      WHERE PPR.IDTPPRODUTOROP = 'OX'
                                                                        AND PPR.CDFILIAL = PR.CDFILIAL
                                                                        AND PPR.CDPRODUTO = PP.CDPRODUTO
                                                                        AND :P_CDARVPROD LIKE PP.CDARVPROD||'%')");
                                                     
            $stmtDE  ->execute($parametros);
            $stmtMO  ->execute($parametros);
            $stmtCO  ->execute($parametros);
            
            $resultDE = $stmtDE ->fetchAll();
            $resultMO = $stmtMO ->fetchAll();
            $resultCO = $stmtCO ->fetchAll();
            
            if(!empty($resultDE)){
                if($resultDE[0]["TIPO"] == "NIVELPOS"){
                    array_push($duplicatedProd, "- O produto  ".$cdarvprod. " - ". $nmproduto." já cadastrado como Despesas Diversas.");
                }else{
                    array_push($duplicatedProd, "- O subnível de produto  ".$cdarvprod. " - ". $nmproduto." já cadastrado como Despesas Diversas.");
                }
            };
            
            if(!empty($resultMO)){
                if($resultMO[0]["TIPO"] == "NIVELPOS"){
                    array_push($duplicatedProd, "- O produto  ".$cdarvprod. " - ". $nmproduto." já cadastrado como despesa de Mão de Obra.");
                }else{
                    array_push($duplicatedProd, "- O subnível de produto  ".$cdarvprod. " - ". $nmproduto." já cadastrado como despesa de Mão de Obra.");
                }
            };
            
            if(!empty($resultCO)){
                if($resultCO[0]["TIPO"] == "NIVELPOS"){
                    array_push($duplicatedProd, "- O produto  ".$cdarvprod. " - ". $nmproduto." já cadastrado como Entrada por Compras.");
                }else{
                    array_push($duplicatedProd, "- O subnível de produto  ".$cdarvprod. " - ". $nmproduto." já cadastrado como Entrada por Compras.");
                }
            };
            
        }else{
            
            $stmtEX = $this->connection->prepare("SELECT PR.CDPRODUTO, P.NMPRODUTO
                                                   FROM PRODUTOROP PR,PRODUTO P
                                                  WHERE PR.CDPRODUTO      = :P_CDPRODUTO
                                                    AND PR.CDFILIAL       = :P_CDFILIAL
                                                    AND PR.IDTPPRODUTOROP = :P_IDTPPRODUTOROP");
            
            $stmtEX  ->execute($parametrosEx);
            $resultEX = $stmtEX ->fetchAll();
            
            if(!empty($resultEX)){
               array_push($duplicatedProd, "Produto  ".$cdarvprod. " - ". $nmproduto." Já cadastrado como ".$tipo);
            }
        }
                          
        return $duplicatedProd;
    }
    
    public function checkRegistros (DTO\Request $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $filtro = array(
            array(
                "columnName" => $rows[0]['CAMPO'],
                "operator" => "=",
                "value" => $rows[0]["CODIGO"]
            ),
            array(
                
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $rows[0]["CDFILIAL"]
            )
        );
        $dataSet= $this->crudService->getFilteredDataSet($rows[0]['TABELA'], $filtro)->getRows();
        $response->addDataSet(new DataSet("RETORNO", $dataSet));
    }
}